<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://t.me/WPTelegram
 * @since      1.0.0
 *
 * @package    Wptelegram
 * @subpackage Wptelegram/admin
 */

/**
 * The admin-specific settings of the plugin.
 *
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wptelegram
 * @subpackage Wptelegram/admin
 * @author     Manzoor Wani <manzoorwani.jk@gmail.com>
 */
class Wptelegram_Admin_Settings {

	/**
	 * Inbuilt and Registered Post Types
	 *
	 * @since  	1.2.0
	 * @access 	private
	 * @var array $post_types Post Types
	 */
	private $post_types;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

	}

    /**
     * get all the settings sections
     *
     * @since    1.2.0
     * @return array settings fields
     */
    public function get_settings_sections() {
        $sections = array(
            array(
                'id'       => 'wptelegram_telegram',
                'title'    => __( 'Telegram Settings', 'wptelegram' ),
                'callback' => array( $this, 'wptelegram_telegram_cb' ),
                'icon_src' => plugins_url( 'icons/telegram.svg' , __FILE__ ),
            ),
            array(
                'id'       => 'wptelegram_wordpress',
                'title'    => __( 'WordPress Settings', 'wptelegram' ),
                'desc'     => __( 'In this section you can teach WordPress how and when to do your job :)', 'wptelegram' ),
                'icon_src' => plugins_url( 'icons/wordpress.svg' , __FILE__ ),
            ),
            array(
                'id'       => 'wptelegram_message',
                'title'    => __( 'Message Settings', 'wptelegram' ),
                'desc'     => __( 'In this section you can change the way messages are sent to Telegram', 'wptelegram' ),
                'icon_src' => plugins_url( 'icons/message.svg' , __FILE__ ),
            )
        );
        return apply_filters('wptelegram_admin_settings_sections_array', $sections);
    }

    /**
     * get all the settings fields
	 *
	 * @since    1.2.0
     * @return array settings fields
     */
    public function get_settings_fields() {
        $settings_fields = array(
            'wptelegram_telegram' => array(
                array(
                    'name'              => 'bot_token',
                    'label'             => __( 'Bot Token', 'wptelegram' ),
                    'desc'              => __( 'Please read the instructions above', 'wptelegram' ),
                    'placeholder'       => __( 'e.g. 123456789:XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX', 'wptelegram' ),
                    'type'              => 'text',
                    'sanitize_callback' => 'wptelegram_sanitize_bot_token',
                    'events'            => array(
                        'onblur' => 'validateToken()',
                    ),
                    'button'            => array(
                        'name'  => __( "Test Token", 'wptelegram' ),
                        'id'    => 'checkbot',
                        'class' => 'button-secondary',
                        'events'=> array(
                            'onclick' => 'getMe()',
                        ),
                    ),
                ),
                array(
                    'name'        => 'bot_html',
                    'desc'        => $this->bot_token_html_cb(),
                    'type'        => 'html'
                ),
                array(
                    'name'              => 'chat_ids',
                    'label'             => __( 'Username(s)/ChatID(s)', 'wptelegram' ),
                    'desc'              => __( 'If more than one, separate them by comma', 'wptelegram' ),
                    'placeholder'       => __( 'e.g. @WPTelegram,-12345678998765', 'wptelegram' ),
                    'type'              => 'text',
                    'sanitize_callback' => 'wptelegram_sanitize_chat_ids',
                    'events'            => array(
                        'onblur' => 'getChatInfo()',
                    ),
                    'button'            => array(
                        'name'  => __('Send Test', 'wptelegram'),
                        'id'    => 'wptelegram-chat-ids-btn',
                        'class' => 'button-secondary',
                        'events'=> array(
                            'onclick' => 'sendMessage()',
                        ),
                    ),
                ),
                array(
                    'name'        => 'chat_html',
                    'desc'        => $this->chat_ids_html_cb(),
                    'type'        => 'html'
                ),
            ),
            'wptelegram_wordpress' => array(
                array(
                    'name'     => 'send_when',
                    'label'    => __( 'When to send?', 'wptelegram' ),
                    'desc'     => __( '', 'wptelegram' ),
                    'type'     => 'multicheck',
                    'as_array' => true,
                    'default'  => 'send_new',
                    'options'  => array(
                        'send_new'     => __( 'When publishing a new post', 'wptelegram' ),
                        'send_updated' => __( 'Updating an existing post', 'wptelegram' ),
                    ),
                    'sanitize_callback' => 'wptelegram_sanitize_array',
                ),
                array(
                    'name'     => 'which_post_type',
                    'label'    => __( 'Which post type(s) to send?', 'wptelegram' ),
                    'desc'     => __( '', 'wptelegram' ),
                    'type'     => 'multicheck',
                    'as_array' => true,
                    'default'  => 'post',
                    'options'  => $this->get_post_types(),
                    'sanitize_callback' => 'wptelegram_sanitize_array',
                ),
                array(
                    'name'     => 'post_edit_switch',
                    'label'    => __( 'When editing a post', 'wptelegram' ),
                    'desc'     => __( 'You can use this switch to override the above settings for a particular post', 'wptelegram' ),
                    'desc_tip' => __( 'Show an ON/OFF switch on the post edit screen', 'wptelegram' ),
                    'default'  => 'on',
                    'type'     => 'checkbox',
                    'sanitize_callback' => 'sanitize_text_field',
                ),
            ),
            'wptelegram_message' => array(
                array(
                    'name'              => 'message_template',
                    'label'             => __( 'Message Template', 'wptelegram' ),
                    'desc'              => __( 'Structure of the message to be sent', 'wptelegram' ),
                    'placeholder'       => __( "e.g.\n{title}\n{full_url}", 'wptelegram' ),
                    'type'              => 'textarea',
                    'sanitize_callback' => 'wptelegram_sanitize_message_template',
                    'json_encoded'      => true,
                    'emoji_container'   => true
                ),
                array(
                    'name'        => 'html',
                    'desc'        => $this->message_template_desc_cb(),
                    'type'        => 'html'
                ),
                array(
                    'name'              => 'excerpt_length',
                    'label'             => __( 'Post Excerpt Length', 'wptelegram' ),
                    'desc'              => __( 'Number of words for the excerpt, to be taken from Post Content.', 'wptelegram' ),
                    'placeholder'       => '55',
                    'min'               => 1,
                    'step'              => '1',
                    'type'              => 'number',
                    'default'           => 55,
                    'sanitize_callback' => 'intval'
                ),
                array(
                    'name'    => 'parse_mode',
                    'label'   => __( 'Parse Mode', 'wptelegram' ),
                    'desc'    => '<a href="'. esc_url( 'https://core.telegram.org/bots/api/#formatting-options' ) . '" target="_blank">' . __( 'Learn more', 'wptelegram' ) . '</a>',
                    'type'    => 'radio',
                    'default' => 'none',
                    'sanitize_callback' => 'sanitize_text_field',
                    'options' => array(
                        'none'      => 'None',
                        'Markdown'  => 'Markdown style',
                        'HTML'      => 'HTML style',
                    ),
                ),
                array(
                    'name'     => 'send_featured_image',
                    'label'    => __( 'Featured Image', 'wptelegram' ),
                    'desc_tip' => __( 'Send Featured Image (if exists)', 'wptelegram' ),
                    'default'  => 'on',
                    'type'     => 'checkbox',
                    'sanitize_callback' => 'sanitize_text_field',
                ),
                array(
                    'name'    => 'image_position',
                    'label'   => __( 'Image Position', 'wptelegram' ),
                    'type'    => 'radio',
                    'sanitize_callback' => 'sanitize_text_field',
                    'default'  => 'before',
                    'options' => array(
                        'before'    => __( 'Before the Text', 'wptelegram' ),
                        'after' => __( 'After the Text', 'wptelegram' ),
                    ),
                ),
                array(
                    'name'     => 'attach_image',
                    'label'    => '',
                    'desc_tip' => __( 'Attach the image below the text', 'wptelegram' ),
                    'type'     => 'checkbox',
                    'sanitize_callback' => 'sanitize_text_field',
                    'desc'    => __( 'Useful if you want everything in one message.<br><b>Note:</b> If enabled:', 'wptelegram' ) . '<ol style="list-style-type:disc;"><li><span id="wptg-attach-caption" class="wptg-attach">' . __( 'Caption cannot be used', 'wptelegram' ) . '</span></li><li><span id="wptg-attach-parse" class="wptg-attach">' . __( 'Parse Mode should NOT be "None".', 'wptelegram' ) . '</span></li><li><span id="wptg-attach-preview" class="wptg-attach">' . __( '"Disable Web Page Preview" should not be checked', 'wptelegram' ) . '</span></li></ol>',
                ),
                array(
                    'name'    => 'image_style',
                    'label'   => __( 'How to send the image?', 'wptelegram' ),
                    'type'    => 'radio',
                    'sanitize_callback' => 'sanitize_text_field',
                    'default'  => 'without_caption',
                    'options' => array(
                        'with_caption'    => __( 'With Caption', 'wptelegram' ),
                        'without_caption' => __( 'Without Caption', 'wptelegram' ),
                    ),
                ),
                array(
                    'name'    => 'caption_source',
                    'label'   => __( 'Caption Source', 'wptelegram' ),
                    'type'    => 'radio',
                    'sanitize_callback' => 'sanitize_text_field',
                    'desc'    => __( 'Telegram limits photo caption to 200 characters.<br>If the Media Library caption is not used and the Message Template exceeds the limit, the remaining part of the Template will be sent separately after the photo without breaking the words.', 'wptelegram' ) .'<br>'. __( '<b>Note: </b> If "Image Position" is set to "After the Text", then "Only Media Library" caption can be used.', 'wptelegram' ),
                    'default' => 'media_library',
                    'options' => array(
                        'media_library'    => __( 'Only Media Library', 'wptelegram' ),
                        'either'           => __( 'Media Library or Message Template (former preferred if not empty)', 'wptelegram' ),
                        'message_template' => __( 'Only Message Template', 'wptelegram' ),
                    ),
                ),
                array(
                    'name'    => 'misc',
                    'label'   => __( 'Link Preview and Notifications', 'wptelegram' ),
                    'desc'    => __( '', 'wptelegram' ),
                    'type'    => 'multicheck',
                    'options' => array(
                        'disable_web_page_preview' => __( 'Disable Web Page Preview (of the link in text)', 'wptelegram' ),
                        'disable_notification'     => __( 'Disable Notifications', 'wptelegram' ),
                    ),
                    'sanitize_callback' => 'wptelegram_sanitize_array',
                ),
            ),
        );
        return apply_filters('wptelegram_admin_settings_fields_array', $settings_fields);
    }

	/**
	 * Render the text for the telegram section
	 *
	 * @since  1.0.0
	 */
	public function wptelegram_telegram_cb() {
		?>
		<div class="inside">
		<p><?php esc_html_e( 'In this section you can change the settings related to Telegram. ', 'wptelegram' );
		 esc_html_e( 'To send messages to a Telegram Channel, group, supergroup or a private chat, follow the steps below.', 'wptelegram' ); ?></p>
		 <p style="color:#f10e0e;text-align:center;"><?php _e( '<b>ATTENTION!</b>','wptelegram'); ?></p>
		 <ol style="list-style-type: decimal;">
		 	<li><?php esc_html_e( 'Create a ', 'wptelegram' ); ?><a href="<?php echo esc_url( 'https://telegram.org/faq_channels#q-what-39s-a-channel' ); ?>" target="_blank"><?php esc_html_e( 'Channel', 'wptelegram' );?></a>/<a href="<?php echo esc_url( 'https://telegram.org/faq#q-how-do-i-create-a-group' ); ?>" target="_blank"><?php esc_html_e( 'group/supergroup', 'wptelegram' );?></a> <?php esc_html_e( '(If you haven\'t)', 'wptelegram' );?></li>
		 	<li><?php _e( 'Create a Bot (If you haven&apos;t), by sending <code>/newbot</code> command to ','wptelegram' );
		 	echo '<a href="https://t.me/BotFather"  target="_blank">@BotFather</a>';?></li>
		 	<li><?php esc_html_e( 'After completing the steps @BotFather will provide you the Bot Token.', 'wptelegram' );?></li>
		 	<li><?php esc_html_e( 'Copy the token and paste into the Bot Token field below. ', 'wptelegram' ); esc_html_e( 'For ease, use ', 'wptelegram' );?><a href="<?php echo esc_url( 'https://web.telegram.org' ); ?>" target="_blank">Telegram Web</a></li>
		 	<li><?php _e( 'Add the Bot as <b>Administrator</b> to your Channel/Group', 'wptelegram' );?></li>
		 	<li><?php esc_html_e( 'Enter the Username(s) or Chat ID(s) separated by comma in the field below', 'wptelegram' );?>
		 		<ol>
		 			<li><?php _e( 'Username must start with <code>@</code>', 'wptelegram' );?></li>
		 			<li><?php esc_html_e( 'You can also use the Chat ID of a private chat with the Bot', 'wptelegram' );?></li>
		 		</ol>
		 	</li>
		 	<li><?php _e( 'Hit <b>Save Changes</b> below', 'wptelegram' );?></li>
		 	<li><?php esc_html_e( 'That\'s it. Happy WPTelagram :)', 'wptelegram' );?></li>
		 </ol>
		 </div>
		<?php
	}

	/**
	 * get bot_token HTML
	 *
	 * @since  1.2.0
	 * @return string
	 */
	private function bot_token_html_cb() {
		$html = '<p style="margin-top:-30px;"><span id="wptelegram-test" class="wptelegram-desc description hidden">' . esc_html__( 'Test result: ', 'wptelegram' ) . '<b><span style="color:#bb0f3b;" id="bot-info"></span></b></span>
		<span id="wptelegram-token-err" class="hidden"  style="color:#f10e0e;font-weight:bold;">' . esc_html__(' Invalid Bot Token', 'wptelegram' ) . '</span></p>';
		return $html;
	}

	/**
	 * get chat_ids HTML
	 *
	 * @since  1.2.0
	 * @return string
	 */
	private function chat_ids_html_cb() {
		$html = '<p style="margin-top:-30px;"><span id="wptelegram-mem-count" class="hidden">' . esc_html__( "Members Count:", "wptelegram" ) . '</span></p>
		<ol id="wptelegram-chat-list">
		</ol>
		<table id="wptelegram-chat-table" class="hidden">
			<tbody>
				<tr>
					<th>' . esc_html__( "Chat_id", "wptelegram" ) . '</th>
					<th>' . esc_html__( "Name/Title", "wptelegram" ) . '</th>
					<th>' . esc_html__( "Chat Type", "wptelegram" ) . '</th>
					<th>' . esc_html__( "Test Status", "wptelegram" ) . '</th>
				</tr>
			</tbody>
		</table>';
		return $html;
	}

	/**
	 * get message template HTML
	 *
	 * @since  1.2.0
	 * @return string
	 */
	private function message_template_desc_cb() {
		$html = '<p style="margin-top:-15px;">' . esc_html__( 'You can use any text, emojis or these macros in any order: ', 'wptelegram' ) . '<b><i>' . esc_html__( '(Click to insert)', 'wptelegram' ) . '</i></b>' . $this->get_macros() . '</p>
			<p>
				<span><strong>' . esc_html__( 'Note:' ) .'</strong></span>
				<ol>
					<li>' . esc_html__( 'Replace ' ) . '<code>taxonomy</code>' . esc_html__( 'in' ) .'<code>{[taxonomy]}</code> ' . esc_html__( 'with the name of the ' ) . '<a href="' . esc_url( 'https://codex.wordpress.org/Taxonomies' ) . '" target="_blank">' . esc_html__( 'taxonomy' ) . '</a>' . esc_html__( ' from which you want to ' ) . '<a href="' . esc_url( 'https://developer.wordpress.org/reference/functions/get_the_terms/' ) . '" target="_blank"> ' . esc_html__( 'get_the_terms' ) . '</a>' . esc_html__( ', attached to the post. For example ' ) . '<code>{[genre]}</code></li>
					<li>' . esc_html__( 'Replace ' ) . '<code>custom_field</code> ' . esc_html__( 'in' ) . ' <code>{[[custom_field]]}</code> ' . __( 'with the name (<code>meta_key</code>) of the ' ) . '<a href="' . esc_url( 'https://codex.wordpress.org/Custom_Fields' ) . ' target="_blank">' . esc_html__( 'Custom Field' ) . '</a>' . esc_html__( ', the value of which you want to add to the template. For example ' ) . '<code>{[[rtl_title]]}</code></li>
				</ol>
			</p>';
			return $html;
	}

    /**
     * get macros
     *
     * @since  1.3.0
     */
    private function get_macros() {
        $macros = array(
            '{ID}',
            '{title}',
            '{author}',
            '{excerpt}',
            '{content}',
            '{short_url}',
            '{full_url}',
            '{tags}',
            '{categories}',
            '{[taxonomy]}',
            '{[[custom_field]]}',
            );

        /**
         * If you add your own macros using this filter
         * You should use "wptelegram_macro_values" filter
         * to replace the macro with the corresponding values
         * See prepare_message() method in
         * wptelegram/includes/class-wptelegram-post-handler.php
         *
         */
        $macros = (array) apply_filters( 'wptelegram_settings_macros', $macros );

        $html = '';
        foreach ( $macros as $macro ) {
            $html .= '<button type="button" class="wptelegram-tag"><code>' . esc_html__( $macro ) . '</code></button>';
        }
        return $html;
    }

    /**
     * Set $this->post_types
     *
     * @since  1.2.0
     */
    public function set_post_types() {
		$this->post_types = get_post_types( array( 'public' => true ), 'objects' );
	}

	/**
	 * get registered post types
	 *
	 * @param  string $for the page or section to get_post_types for
	 *
	 * @since  1.2.0
	 * @return array
	 */
	public function get_post_types( $for = 'options' ) {
		$arr = array();
		if ( 'metabox' == $for ) {
			foreach ( $this->post_types  as $post_type ) {
                if ( 'attachment' != $post_type->name ){
                    $arr[] = $post_type->name;
                }
			}
		} else{
			foreach ( $this->post_types  as $post_type ) {
                if ( 'attachment' != $post_type->name ){
        			$arr[ $post_type->name ] = isset( $post_type->labels->singular_name ) ? $post_type->labels->singular_name . ' (' . $post_type->name . ')' : $post_type->name;
                }
			}
		}
		return $arr;
	}

	/**
	 * Render the meta box
	 *
	 * @since  1.0.0
	 */
	public function wptelegram_meta_box_cb() {
		global $post;
		wp_nonce_field( 'save_scheduled_post_meta', 'wptelegram_meta_box_nonce' );

		$chat_ids = wptelegram_get_option( 'chat_ids', 'telegram' );

	    $message_template = wptelegram_get_option( 'message_template', 'message' );

		if ( '' != $message_template ) {
			$message_template = json_decode( $message_template );
		}
		$message_template = apply_filters( 'wptelegram_message_template', $message_template, $post );
		?>
		<input type="checkbox" name="<?php echo esc_attr( 'wptelegram_override_switch'); ?>" id="<?php echo esc_attr( 'wptelegram_override_switch' ); ?>" value="on">
		<label for="<?php echo esc_attr( 'wptelegram_override_switch' ); ?>" id="<?php echo esc_attr( 'wptelegram_override_switch-label' ); ?>"></label><label for="<?php echo esc_attr( 'wptelegram_override_switch' ); ?>" id="switch-label"><?php esc_html_e( ' Override default settings', 'wptelegram' ); ?></label>
		<table class="form-table" id="wptelegram-meta-table">
			<tbody>
				<tr><th scope="row"><?php esc_html_e( 'Message', 'wptelegram' ); ?></th>
					<td>
						<fieldset>
							<label>
								<input type="radio" name="<?php echo esc_attr( 'wptelegram_send_message' ); ?>" id="<?php echo esc_attr( 'wptelegram_send_message' ); ?>" value="yes" checked><?php esc_html_e( 'Send', 'wptelegram' ); ?>
							</label>
							<br>
							<label>
								<input type="radio" name="<?php echo esc_attr( 'wptelegram_send_message' ); ?>" value="no"><?php esc_html_e( 'Do not Send', 'wptelegram' ); ?>
							</label>
						</fieldset>
					</td>
				</tr>
				<tr id="send_to"><th scope="row"><?php esc_html_e( 'Send to', 'wptelegram' ); ?></th>
					<td>
					<?php
					if ( $chat_ids ) : ?>
						<fieldset>
							<label>
								<input type="checkbox" id="<?php echo esc_attr( 'wptelegram_send_to_all' ); ?>" name="<?php echo esc_attr( 'wptelegram_send_to[all]' ); ?>" value="1" checked><?php esc_html_e( 'All Channels/Chats', 'wptelegram' ); ?>
							</label>
							<?php
							$chat_ids = explode( ',', $chat_ids );
							foreach ( $chat_ids as $chat_id ) :
								if ( '' == $chat_id ):
									continue;
								endif; ?>
								<br>
								<label class="<?php echo esc_attr( 'wptelegram_send_to' ); ?>" >
									<input type="checkbox" name="<?php echo esc_attr( 'wptelegram_send_to[]' ); ?>" value="<?php echo $chat_id; ?>"><?php echo $chat_id; ?>
								</label>
							<?php endforeach; ?>
						</fieldset>
					<?php else : ?>
						<span><?php esc_html_e( 'No Channels/Chat IDs found', 'wptelegram' ); ?></span>
					<?php endif; ?>
					</td>
				</tr>
				<tr id="message_template"><th scope="row"><?php esc_html_e( 'Template', 'wptelegram' ); ?></th>
					<td>
                        <div id="<?php echo esc_attr( 'wptelegram_message_template-container' ); ?>"></div>
						<textarea id="<?php echo esc_attr( 'wptelegram_message_template' ); ?>" name="<?php echo esc_attr( 'wptelegram_message_template' ); ?>" dir="auto"><?php echo esc_textarea( $message_template ); ?></textarea>
						<br><br>
						<?php
						echo call_user_func( array( $this, 'message_template_desc_cb' ) );
						?>
					</td>
				</tr>
			</tbody>
		</table>
	    <?php
	}

}
