<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://t.me/WPTelegram
 * @since      1.3.7
 *
 * @package    Wptelegram
 * @subpackage Wptelegram/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wptelegram
 * @subpackage Wptelegram/admin
 * @author     Manzoor Wani <manzoorwani.jk@gmail.com>
 */
class Wptelegram_Admin {

	/**
	 * Title of the plugin.
	 *
	 * @since    1.3.7
	 * @access   protected
	 * @var      string    $title    Title of the plugin
	 */
	protected $title;

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

    /**
     * WPTelegram_Settings_API
     *
     * @var Object
     */
    private $settings_api;

	/**
	 * Object which handles settings sections, fields and metabox
	 *
	 * @since  	1.0.0
	 * @access 	private
	 * @var Wptelegram_Admin_Settings $tg_api Admin Settings Object
	 */
	private $settings;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $title, $plugin_name, $version ) {

		$this->title = $title;
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->settings = new Wptelegram_Admin_Settings();
		$this->settings_api = new WPTelegram_Settings_API( $plugin_name );
		if ( get_option( 'wptelegram_bot_token' ) ) {
			$this->update_options();
		}
	}

	/**
	 * Update Options after upgrade from version < 1.3.0
	 *
	 * @since    1.3.0
	 */
	private function update_options() {
		// telegram options array
		$telegram = array(
			'bot_token' => get_option( 'wptelegram_bot_token', '' ),
			'chat_ids'  => get_option( 'wptelegram_chat_ids', '' ),
		);

		// wordpress options array
		$wordpress = array();
		// When to send
		$send_when = array();
		if ( '1' == get_option( 'wptelegram_send_new' ) ) {
			$send_when[] =  'send_new';
		}
		if ( '1' == get_option( 'wptelegram_send_updated' ) ) {
			$send_when[] = 'send_updated';
		}
		// push to wordpress options if non empty
		if ( ! empty( $send_when ) ) {
			$wordpress['send_when'] = $send_when;
		}

		// Which post types to send
		$which_post_type = array();
		if ( '1' == get_option( 'wptelegram_send_post' ) ) {
			$which_post_type[] =  'post';
		}
		if ( '1' == get_option( 'wptelegram_send_page' ) ) {
			$which_post_type[] = 'page';
		}
		if ( '1' == get_option( 'wptelegram_send_cpt' ) ) {
			if ( get_option( 'wptelegram_cpt' ) ) {
				$cpt = explode( ',', get_option( 'wptelegram_cpt' ) );
				// remove empty values
				$cpt = array_filter( $cpt, 'strlen' );
				$which_post_type = array_merge( $which_post_type, $cpt);
			}
		}
		// push to wordpress options if non empty
		if ( ! empty( $which_post_type ) ) {
			$wordpress['which_post_type'] = $which_post_type;
		}

		// Post edit switch
		if ( '1' == get_option( 'wptelegram_post_edit_switch' ) ) {
			$post_edit_switch =  'on';
		} else{
			$post_edit_switch =  'off';
		}
		// push to wordpress options
		$wordpress['post_edit_switch'] = $post_edit_switch;

		// message options array
		$message = array();
		$message_template = get_option( 'wptelegram_message_template', '' );
		if ( '' != $message_template ) {
			$message_template =  json_decode( $message_template );
			$message_template = json_encode( $message_template->text );
		}
		// push to message options
		$message['message_template'] = $message_template;

		// excerpt length
		$excerpt_length = get_option( 'wptelegram_excerpt_length', '55' );
		// push to message options
		$message['excerpt_length'] = $excerpt_length;

		// Send featured image
		if ( '1' == get_option( 'wptelegram_send_featured_image' ) ) {
			$send_featured_image =  'on';
		} else{
			$send_featured_image =  'off';
		}
		// push to message options
		$message['send_featured_image'] = $send_featured_image;

		// parse mode
		$parse_mode = get_option( 'wptelegram_parse_mode', 'none' );
		// push to message options
		$message['parse_mode'] = $parse_mode;

		// miscellaneous options
		$misc = array();
		if ( '1' == get_option( 'wptelegram_disable_web_page_preview' ) ) {
			$misc['disable_web_page_preview'] =  'disable_web_page_preview';
		}
		if ( '1' == get_option( 'wptelegram_disable_notification' ) ) {
			$misc['disable_notification'] = 'disable_notification';
		}
		// push to message options if non empty
		if ( ! empty( $misc ) ) {
			$message['misc'] = $misc;
		}

		// create new options
		$new_optons = array(
			'wptelegram_telegram'  => $telegram,
			'wptelegram_wordpress' => $wordpress,
			'wptelegram_message'   => $message,
		);
		// add new options to database
		foreach ( $new_optons as $name => $value ) {
			update_option( $name, $value, true );
		}
		
		// old options
		$old_options = array(
			'bot_token',
			'chat_ids',
			'send_new',
			'send_updated',
			'send_post',
			'send_page',
			'send_cpt',
			'cpt',
			'post_edit_switch',
			'default_send',
			'message_template',
			'excerpt_length',
			'send_featured_image',
			'parse_mode',
			'message_formatting',
			'disable_web_page_preview',
			'disable_notification',
		);
		// delete old options
		foreach ( $old_options as $option ) {
			delete_option( 'wptelegram_' . $option );
		}
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wptelegram-admin.css', array(), $this->version, 'all' );
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/wptelegram-admin.js', array( 'jquery' ), $this->version, false );
	}

    /**
     * Register settings
     *
     * @since  1.3.0
     */
    public function admin_init() {

        /**
		 * use admin_init hook to set post types
		 * because add_meta_boxes hook would not be able to
		 * return Custom Post Types
		 */
		$this->settings->set_post_types();
        //set the settings
        $this->settings_api->set_sections( $this->settings->get_settings_sections() );
        $this->settings_api->set_fields( $this->settings->get_settings_fields() );

        //initialize settings
        $this->settings_api->admin_init();
    }

	/**
	 * Add an entry in the Admin Menu
	 *
	 * @since  1.0.0
	 */
	public function add_menu_page() {

		add_menu_page(
			__( 'WP Telegram Settings', 'wptelegram' ),
			__( 'WP Telegram', 'wptelegram' ),
			'manage_options',
			$this->plugin_name,
			array( $this, 'display_settings_page' ),
			plugins_url( '/icons/icon-16x16-white.svg', __FILE__ )
		);
	
	}

	/**
	 * Render the menu page for plugin
	 *
	 * @since  1.3.7
	 */
	public function display_settings_page() {
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( __( 'Sorry, you are not allowed to access this page.' ) );
		}
        $this->load_emojioneArea();
		echo '<div class="wrap wptelegram" id="wptelegram-wrap">';

        include_once 'partials/wptelegram-admin-header.php';

        $this->settings_api->show_navigation();
        $this->settings_api->show_forms();
        echo '</div>';
        // the template containing js
        include_once 'partials/wptelegram-admin-display.php';
	}

	/**
	 * Register Metabox
	 *
	 * @since  1.0.0
	 */
	public function register_meta_box()
	{
		$post_edit_switch = wptelegram_get_option( 'post_edit_switch', 'wordpress' );
		if ( 'off' == $post_edit_switch ) {
			return;
		}

        $this->load_emojioneArea();

		$screens = $this->settings->get_post_types( 'metabox' );
	    $screens = (array) apply_filters( 'wptelegram_cpt_meta_box_screens', $screens);

	    /**
	     * The $screen parameter as array to add_meta_box
	     * was added in WordPress 4.4
	     * So, passing as string to support older versions
	     */
	    foreach ( $screens as $screen ) {
	    	add_meta_box(
	            'wptelegram_meta_box',
	            __( 'WP Telegram Options', 'wptelegram' ),
	            array( $this->settings, 'wptelegram_meta_box_cb' ),
	            $screen,
	            'normal',
	            'high'
	        );
	    }
        wp_enqueue_style( $this->plugin_name.'-metabox', plugin_dir_url( __FILE__ ) . 'css/wptelegram-admin-metabox.css', array(), $this->version, 'all' );
	}

	/**
	 * Load Emoji One Area
	 *
	 * @since  1.3.0
	 */
	private function load_emojioneArea(){
		wp_enqueue_style( $this->plugin_name.'-emojicss', plugin_dir_url( __FILE__ ) . 'emoji/emojionearea.min.css', array(), $this->version, 'all' );
        wp_enqueue_script( $this->plugin_name.'-emojijs', plugin_dir_url( __FILE__ ) . 'emoji/emojionearea.min.js', array(), $this->version, 'all' );
	}

	/**
	 * Show admin notices on failure
	 *
	 * @since  1.3.0
	 */
	public function admin_notices() {
		if ( ! isset( $_GET['wptelegram'] ) ) {
		 return;
		}
		?>
		<?php
		$code = $_GET['wptelegram'];
		$message = __( 'WP Telegram failed to communicate.', 'wptelegram' );
		if ( $error = get_site_transient( 'wptelegram_http_error' ) ) {
			$message .= ' <b>' . __( 'Reason: ', 'wptelegram' ) . '</b>' . $error->get_error_message();
			delete_transient( 'wptelegram_http_error' );
		}
		?>
		<div class="notice notice-error is-dismissible">
		  <p><b><?php esc_html_e( 'Error: ', 'wptelegram'); _e( $code ); ?>&#33;</b>&nbsp;<?php _e( $message ); ?></p>
		</div>
		<?php
	}

}
